
import torch
from torch_cfd.grids import *
from torch_cfd.equations import *
from torch_cfd.initial_conditions import *
from torch_cfd.finite_differences import *
from torch_cfd.forcings import *
from data_gen import *

device = torch.device("cuda" if torch.cuda.is_available() else "cpu")


def main():
    diam = 2 * torch.pi
    n = 256
    dt = 1e-3
    viscosity = 1e-3
    peak_wavenumber = 4

    grid = Grid(shape=(n, n), domain=((0, diam), (0, diam)), device=device)

    ns2d = NavierStokes2DSpectral(
            viscosity=viscosity,
            grid=grid,
            drag=0,
            smooth=True,
            forcing_fn=None,
            solver=rk4_crank_nicolson,
        ).to(device)

    vort_init = torch.stack([
                    vorticity_field(
                        grid, peak_wavenumber, random_state=42
                    ).data
                    for k in range(1)
                ]
            )
    vort_hat = fft.rfft2(vort_init).to(device)

    _ = get_trajectory_rk4(
            ns2d,
            vort_hat,
            dt,
            num_steps=1000,
            record_every_steps=1,
            pbar=True,
        )

if __name__ == "__main__":
    main()
